searchNodes=[{"doc":"RulEx is a rules engine and simplified expressions language for evaluating a set of conditions against a set of facts, yielding true or false on whether the provided facts adhere to the set of conditions given. RulEx expressions The expressions used by RulEx are inspired by Lisp, and are simply nested lists with the first element being an operand, think of it as a function, and the remaining elements arguments to this operand. Evaluating these expressions is done against a set of facts provided as a RulEx.DataBag , and the outcome is a boolean on whether or not the conditions match on the given facts. Supported operands Logical operands These operands can only be used in RulEx.Behaviour.eval/2 callback, and only yield true or false results. They can be passed &quot;facts&quot; from outside the expressions using RulEx.DataBag and the value operands supported by RulEx. The any operand :| , &quot;|&quot; , which matches any list of RulEx expressions and yields true if one of them yields true, false otherwise. The all operand :&amp; , &quot;&amp;&quot; , which matches any list of RulEx expressions and yields true if non of them yields false, true otherwise. The negation operand :! , which matches a single RulEx expressions and yields the negation of whatever the input expression yields, e.g. true for false, and false for true. The equality operand := , &quot;=&quot; , which matches a two RulEx value expressions and yields true if they are equal, this operation is non strict, i.e. 1.0 = 0 is true. The inequality operand :!= , &quot;!=&quot; , which matches a two RulEx value expressions and yields true if they are not equal, this operation is non strict, i.e. 1.0 != 0 is false. The less than operand :&lt; , &quot;&lt;&quot; , which matches a two RulEx value expressions and yields true if the one on the left hand side is less than the one on the right hand side. The greater than operand :&gt; , &quot;&gt;&quot; , which matches a two RulEx value expressions and yields true if the one on the left hand side is greater than the one on the right hand side. The less than or equals operand :&lt;= , &quot;&lt;=&quot; , which matches a two RulEx value expressions and yields true if the one on the left hand side is less than or equals the one on the right hand side. The greater than or equals operand :&gt;= , &quot;&gt;=&quot; , which matches a two RulEx value expressions and yields true if the one on the left hand side is greater than or equals the one on the right hand side. The contains operand :in , &quot;in&quot; , which matches a &quot;needle&quot;, a single RulEx value expressions, and a &quot;haystack&quot;, a list of normal values (not RulEx expressions), and yields true if the needle exists in the given haystack. Value operands Value operands are used to represent data in a RulEx expressions, they are only two operands for this use case :val ( &quot;val&quot; ) and :var ( &quot;var&quot; ). val operand can be used in order to store exact values in the expressions, this includes the conditions you want to mean when applying the logical operands. var operand can be used in order to pass facts from outside of the expressions, this is done by using any Elixir term that implements the RulEx.DataBag protocol. Both these operands accept 2 arguments, a data type, and a value, the value in a val expressions is the actual value to be yielded back, while in a var expressions is they key used with RulEx.DataBag . The values are validated against the data type argument given, and would fail/be rejected if they don't match properly, i.e. you cannot return the value &quot;string&quot; when the data type expressed is &quot;numeric&quot; . The supported data types are, &quot;any&quot; , which will yield back any value without any validation. &quot;number&quot; , which will yield back any numeric value, regardless if an integer or float. &quot;integer&quot; , which will yield back only integer values. &quot;float&quot; , which will yield back only float values. &quot;string&quot; , which will yield back only strings. &quot;boolean&quot; , which will yield back only boolean values, i.e. true and false . &quot;list&quot; , which will yield back only lists of values, no validation is required on the values within the list. &quot;map&quot; , which will yield back only map of arbitrary keys and values, no validation is required on either the keys or the values within the map. &quot;time&quot; , which will yield back time values, it can be given string values and it will parse them, so long as those values are times as defined by ISO 8601. &quot;date&quot; , which will yield back date values, it can be given string values and it will parse them, so long as those values are dates as defined by ISO 8601. &quot;datetime&quot; , which will yield back date time values (naive datetime in Elixir terminology), it can be given string values and it will parse them, so long as those values are datetimes as defined by ISO 8601. Currently RulEx does not support any other additional types, and all of it's operations regarding values are strictly typed, meaning it will reject to complete operations if types don't match. Reserved operands RulEx reserves all the previously defined expressions as defined in Logical operands and in Value operands , in both their string and atom formats. RulEx also disallows for any non-reserved operand that isn't a string. RulEx does however allow for adding custom operands as defined in Custom operand. Custom operands RulEx behaviour can be extended with any arbitrary custom operands by simply defining the RulEx.Behaviour.opernad/3 callback, this will receive the operand string, a list of arguments given to the operand, and the current data bag holding the facts being processed. Example expressions You can view the test suite for example expressions of a variety of types and forms, in short an expression is simply a list of arbitrarily nested RulEx valid values, this includes other RulEx expressions and normal Elixir terms, e.g. [:=, [:val, &quot;any&quot;, &quot;hello&quot;], [:val, &quot;any, &quot;world&quot;]] is a valid expression. Think of RulEx expressions very similarly to how Lisp syntax works, these can be built and manipulated by code easily due to their easy to use structure. Some examples are given below, for more varied examples please check the test suite! [ :! , [ := , [ :val , &quot;string&quot; , &quot;some value&quot; ] , [ :val , &quot;string&quot; , &quot;some other value&quot; ] ] ] [ :| , [ := , [ :val , &quot;string&quot; , &quot;some value&quot; ] , [ :val , &quot;string&quot; , &quot;some other value&quot; ] ] , [ := , [ :val , &quot;numeric&quot; , 10 ] , [ :val , &quot;numeric&quot; , 10.0 ] ] ] Storing and transporting these expressions RulEx provides the RulEx.EncodeError behaviour used to define converting RulEx expressions from and to Elixir terms, this can be helpful when you need to store these expressions and/or to transfer these expressions over the wire. By default, a JSON encoding is implemented for you. Usage Simply use RulEx as is, as it implements the RulEx.Behaviour fully. However, if you wan to add custom operands to the set of supported rules, simply define the RulEx.Behaviour as is, and then run wild. To do so simply use RulEx.Behaviour in your module, then implement your custom operands by overriding the RulEx.Behaviour.operand/3 callback. Caveats and quirks If no arguments are given to the any ( :| ) operand, it will yield back false , follow this issue in Elixir for a discussion around this behaviour . If no arguments are given to the all ( :&amp; ) operand, it will yield back true . Value expressions can be used in the RulEx.Behaviour.eval/2 callback, the results will simply be converted based on their truthiness as defined by Elixir and Erlang (i.e. only false and nil are falsy values). Logical expressions will always yield an error if passed to the RulEx.Behaviour.value/2 callbacks. Results coming back from custom RulEx.Behaviour.operand/3 are treated like value expressions and are converted to booleans based on their truthiness. If given a reserved expression but as a string instead of an atom, RulEx will convert it to an atom and use it as if it was passed as the atom for the reserved expression. The comparison operands, &lt; , &gt; , &lt;= , &gt;= , = , and != all validate that both arguments are val or var expressions of the same type before doing anything, and will yield an error otherwise.","ref":"RulEx.html","title":"RulEx","type":"module"},{"doc":"Default implementation for RulEx.Encoding.decode/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.html#decode/1","title":"RulEx.decode/1","type":"function"},{"doc":"Default implementation for RulEx.Encoding.decode!/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.html#decode!/1","title":"RulEx.decode!/1","type":"function"},{"doc":"Default implementation for RulEx.Encoding.encode/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.html#encode/1","title":"RulEx.encode/1","type":"function"},{"doc":"Default implementation for RulEx.Encoding.encode!/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.html#encode!/1","title":"RulEx.encode!/1","type":"function"},{"doc":"Default implementation for RulEx.Behaviour.eval/2 . Examples iex&gt; import Elixir.RulEx iex&gt; # Success cases iex&gt; truthy_expression = [ := , [ :val , &quot;string&quot; , &quot;hello&quot; ] , [ :var , &quot;string&quot; , &quot;what?&quot; ] ] iex&gt; falsy_expression = [ := , [ :val , &quot;string&quot; , &quot;hello&quot; ] , [ :val , &quot;string&quot; , &quot;world&quot; ] ] iex&gt; { :ok , true } = eval ( truthy_expression , %{ &quot;what?&quot; =&gt; &quot;hello&quot; } ) iex&gt; { :ok , false } = eval ( falsy_expression , %{ &quot;what?&quot; =&gt; &quot;hello&quot; } ) iex&gt; invalid_expression = [ ] iex&gt; { :error , _reason } = eval ( invalid_expression , %{ } )","ref":"RulEx.html#eval/2","title":"RulEx.eval/2","type":"function"},{"doc":"Default implementation for RulEx.Behaviour.eval!/2 .","ref":"RulEx.html#eval!/2","title":"RulEx.eval!/2","type":"function"},{"doc":"Default implementation for RulEx.Behaviour.expr?/1 . Examples iex&gt; import Elixir.RulEx iex&gt; correct_expression = [ := , [ :val , &quot;string&quot; , &quot;hello&quot; ] , [ :var , &quot;string&quot; , &quot;what?&quot; ] ] iex&gt; incorrect_expression = [ ] iex&gt; true = expr? ( correct_expression ) iex&gt; false = expr? ( incorrect_expression )","ref":"RulEx.html#expr?/1","title":"RulEx.expr?/1","type":"function"},{"doc":"Default implementation for RulEx.Behaviour.operand/3 . If not overridden this will always yield an error. No catchall clause is needed as it is already implemented. Examples iex&gt; import Elixir.RulEx iex&gt; { :error , _reason } = operand ( &quot;whatever&quot; , &quot;any value&quot; , %{ } )","ref":"RulEx.html#operand/3","title":"RulEx.operand/3","type":"function"},{"doc":"Default implementation for RulEx.Behaviour.value/2 . Examples iex&gt; import Elixir.RulEx iex&gt; # Success cases iex&gt; val_expression = [ :val , &quot;string&quot; , &quot;foo&quot; ] iex&gt; var_expression = [ :var , &quot;number&quot; , &quot;x&quot; ] iex&gt; { :ok , &quot;foo&quot; } = value ( val_expression , %{ } ) iex&gt; { :ok , 10 } = value ( var_expression , %{ &quot;x&quot; =&gt; 10 } ) iex&gt; # Error cases iex&gt; { :error , _reason } = value ( var_expression , %{ } ) iex&gt; { :error , _reason } = value ( [ ] , %{ } )","ref":"RulEx.html#value/2","title":"RulEx.value/2","type":"function"},{"doc":"Default implementation for RulEx.Behaviour.value!/2 .","ref":"RulEx.html#value!/2","title":"RulEx.value!/2","type":"function"},{"doc":"The set of all valid values that represent an &quot;argument&quot; in a RulEx expression.","ref":"RulEx.html#t:arg/0","title":"RulEx.arg/0","type":"type"},{"doc":"The set of all supported operands by RulEx, these include a set of reserved operands, as well as any arbitrary strings that can be used to extend RulEx behaviour to match any custom domain that needs rules evaluation.","ref":"RulEx.html#t:op/0","title":"RulEx.op/0","type":"type"},{"doc":"A RulEx expression is a list of operands and arguments, ideally these expressions must start with one operand and then be followed by any arbitrarily long set of arguments, the arguments can also be other RulEx expressions. In order to validate whether a given RulEx expression is valid or not you can use the callback RulEx.Behaviour.expr?/1 . Operands and expressions can further be validated with the guards defined in the RulEx.Guards module.","ref":"RulEx.html#t:t/0","title":"RulEx.t/0","type":"type"},{"doc":"This is the main behaviour describing RulEx and all the available callbacks and functions needed to fully implement rules evaluation. Optionally, using RulEx.Behaviour to generate your custom RulEx module can also implement the RulEx.Encoding behaviour. Usage A custom RulEx module can be defined by simply using RulEx.Behaviour defmodule MyApp.Rules do use RulEx.Behaviour end When using the default set of rules defined by RulEx, this is not needed as a default implementing already exists in RulEx . Extending RulEx operands RulEx behaviour provides the ability to extend it's supported operands arbitrarily via the operand/3 callback, by default this will yield an error for all non reserved operands as defined by the type RulEx.op . defmodule MyApp.CustomRules do use RulEx.Behaviour # Here we&#39;re matching against our custom operand name # afterwards we assert that the single expression # given to us will yield `nil`. # # A catchall operand for all non-implemented operands # exists and will yield back an error. # def operand ( &quot;nil?&quot; , [ arg ] , db ) do with value &lt;- value ( arg , db ) , do : { :ok , is_nil ( value ) } end end Defining a default encoding mechanism RulEx behaviour optionally allows you to provide a custom encoding mechanism to translate RulEx expression into whatever encoding you want/need. This is achieve by either setting the application environment to define a config for under RulEx.Behaviour key with a keyword list with :default key set to the encoding module (implementing the behaviour RulEx.Encoding ). # In your `config/config.exs` config :rul_ex , RulEx.Behaviour , default : MyApp.RulEx.Encoder Further more, when defining the module you can pass encoder option when using this behaviour as such. defmodule MyApp.CustomRules do use RulEx.Behaviour , encoder : MyApp.RulEx.Encoder end Do note that passing the :encoder option overrides any application configurations set in the environment. You can disable the encoder behaviour all together by passing the option :without_encoder when using this behaviour, as such. defmodule MyApp.CustomRules do use RulEx.Behaviour , :without_encoder end If no explicit encoding mechanism is provided, an the option :without_encoder isn't passed, then using this behaviour to build your custom RulEx module will use RulEx.Encoding.Json as an encoder to implement the RulEx.Encoding behaviour.","ref":"RulEx.Behaviour.html","title":"RulEx.Behaviour","type":"behaviour"},{"doc":"Given a RulEx expression (defined by the type RulEx.t ), and a databag holding any contextual variable info (facts, implementing the protocol RulEx.DataBag ), evaluate the expression against the databag and yield the result. This function will return an error if the expression given to it is an expression for the operands :val or :var , as those expressions yield any arbitrary value instead of a boolean result defining whether the RulEx expression is truthy or falsy given the facts provided (via the databag).","ref":"RulEx.Behaviour.html#c:eval/2","title":"RulEx.Behaviour.eval/2","type":"callback"},{"doc":"Exactly identical to RulEx.Behaviour.eval/2 but raises RulEx.EvalError in case of errors.","ref":"RulEx.Behaviour.html#c:eval!/2","title":"RulEx.Behaviour.eval!/2","type":"callback"},{"doc":"Validate that the given term is a valid RulEx expression as defined by RulEx.t .","ref":"RulEx.Behaviour.html#c:expr?/1","title":"RulEx.Behaviour.expr?/1","type":"callback"},{"doc":"This function can be used to extend RulEx' defined operands arbitrarily by behaviour implementors. This function will never execute for any of the RulEx reserved operands, as defined by the type RulEx.op .","ref":"RulEx.Behaviour.html#c:operand/3","title":"RulEx.Behaviour.operand/3","type":"callback"},{"doc":"Given a RulEx value expression (defined by the type RulEx.t of operands :val or :var ), and a databag holding any contextual variable info (facts, implementing the protocol RulEx.DataBag ), evaluate the value expression against the databag and yield the result. This function will return an error if the given any expression except those of the operands :val or :var . If the resolved value does not match the type specified in the expression this function will yield back an error. If given a :var expression, this function should yield back an error if the databag doesn't hold a value for the requested variable. This function will yield an error if given any expression that isn't a RulEx :var or :var expression, regardless of its correctness as a RulEx expression.","ref":"RulEx.Behaviour.html#c:value/2","title":"RulEx.Behaviour.value/2","type":"callback"},{"doc":"Exactly identical to RulEx.Behaviour.value/2 but raises RulEx.EvalError in case of errors.","ref":"RulEx.Behaviour.html#c:value!/2","title":"RulEx.Behaviour.value!/2","type":"callback"},{"doc":"Protocol controlling providing facts to RulEx evaluation functions. Examples defimpl RulEx.DataBag , for : Keyword do def get ( kw , key , default \\\\ nil ) do Keyword . get ( kw , key , default ) end end","ref":"RulEx.DataBag.html","title":"RulEx.DataBag","type":"protocol"},{"doc":"Given an term implementing the RulEx.DataBag behaviour, a key, and an optional default value, yield back the value found in the provided databag. Examples iex&gt; 10 = RulEx.DataBag . get ( %{ &quot;x&quot; =&gt; 10 } , &quot;x&quot; ) iex&gt; 10 = RulEx.DataBag . get ( %{ &quot;x&quot; =&gt; 10 } , &quot;x&quot; , 11 ) iex&gt; 11 = RulEx.DataBag . get ( %{ &quot;y&quot; =&gt; 10 } , &quot;x&quot; , 11 )","ref":"RulEx.DataBag.html#get/3","title":"RulEx.DataBag.get/3","type":"function"},{"doc":"","ref":"RulEx.DataBag.html#t:t/0","title":"RulEx.DataBag.t/0","type":"type"},{"doc":"This behaviour defines how to translate RulEx expressions into any encoding formats your application may want/need, e.g. converting them into JSON values. This is useful when you want to store RulEx expressions into database and/or if you want to transfer the rules over the wire to other services/systems that may need it. Usage A custom RulEx encoding mechanism can be defined by simply using RulEx.Encoding , and passing a module defining your encode and decode functions defmodule MyApp.RulEx.Encoder do use RulEx.Encoding , encoder : CustomModule end Alternatively the internal encoder can be passed as an option when using RulEx.Encoding defmodule MyApp.RulEx.Encoder do use RulEx.Encoding def __encoder__ , do : CustomModule end Using this module to build your encoder is useful as the generated encoder would do validation on the RulEx expressions being encoded or decoded by it.","ref":"RulEx.Encoding.html","title":"RulEx.Encoding","type":"behaviour"},{"doc":"Given an encoded RulEx expression, decode it back into a RulEx expression. This function will yield an error if decoded value is an invalid RulEx expression, or if it fails to decode the provided value. Examples iex&gt; # Assuming the use of a JSON encoder iex&gt; encoded = &quot;[ \\&quot; | \\&quot; ,[ \\&quot; &gt; \\&quot; ,[ \\&quot; val \\&quot; , \\&quot; number \\&quot; ,10],[ \\&quot; var \\&quot; , \\&quot; number \\&quot; , \\&quot; x \\&quot; ]],[ \\&quot; = \\&quot; ,[ \\&quot; val \\&quot; , \\&quot; any \\&quot; ,10],[ \\&quot; var \\&quot; , \\&quot; any \\&quot; , \\&quot; x \\&quot; ]]]&quot; iex&gt; expression = [ :| , [ :&gt; , [ :val , &quot;number&quot; , 10 ] , [ :var , &quot;number&quot; , &quot;x&quot; ] ] , [ := , [ :val , &quot;any&quot; , 10 ] , [ :var , &quot;any&quot; , &quot;x&quot; ] ] ] iex&gt; { :ok , ^ expression } = decode ( encoded ) iex&gt; { :error , _reason } = decode ( &quot;[]&quot; )","ref":"RulEx.Encoding.html#c:decode/1","title":"RulEx.Encoding.decode/1","type":"callback"},{"doc":"Exactly identical to RulEx.Encoding.decode/1 but raises RulEx.DecodeError in case of errors.","ref":"RulEx.Encoding.html#c:decode!/1","title":"RulEx.Encoding.decode!/1","type":"callback"},{"doc":"Given a RulEx expression, encode it into any parsable value. This function will yield an error if given an invalid value in place of the RulEx expression, or if it fails to encode the provided expression. Examples iex&gt; # Assuming the use of a JSON encoder iex&gt; expression = [ :| , [ :&gt; , [ :val , &quot;number&quot; , 10 ] , [ :var , &quot;number&quot; , &quot;x&quot; ] ] , [ := , [ :val , &quot;any&quot; , 10 ] , [ :var , &quot;any&quot; , &quot;x&quot; ] ] ] iex&gt; encoded = &quot;[ \\&quot; | \\&quot; ,[ \\&quot; &gt; \\&quot; ,[ \\&quot; val \\&quot; , \\&quot; number \\&quot; ,10],[ \\&quot; var \\&quot; , \\&quot; number \\&quot; , \\&quot; x \\&quot; ]],[ \\&quot; = \\&quot; ,[ \\&quot; val \\&quot; , \\&quot; any \\&quot; ,10],[ \\&quot; var \\&quot; , \\&quot; any \\&quot; , \\&quot; x \\&quot; ]]]&quot; iex&gt; { :ok , ^ encoded } = encode ( expression ) iex&gt; { :error , _reason } = encode ( [ ] )","ref":"RulEx.Encoding.html#c:encode/1","title":"RulEx.Encoding.encode/1","type":"callback"},{"doc":"Exactly identical to RulEx.Encoding.encode/1 but raises RulEx.EncodeError in case of errors.","ref":"RulEx.Encoding.html#c:encode!/1","title":"RulEx.Encoding.encode!/1","type":"callback"},{"doc":"","ref":"RulEx.Encoding.Json.html","title":"RulEx.Encoding.Json","type":"module"},{"doc":"Default implementation for RulEx.Encoding.decode/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.Encoding.Json.html#decode/1","title":"RulEx.Encoding.Json.decode/1","type":"function"},{"doc":"Default implementation for RulEx.Encoding.decode!/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.Encoding.Json.html#decode!/1","title":"RulEx.Encoding.Json.decode!/1","type":"function"},{"doc":"Default implementation for RulEx.Encoding.encode/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.Encoding.Json.html#encode/1","title":"RulEx.Encoding.Json.encode/1","type":"function"},{"doc":"Default implementation for RulEx.Encoding.encode!/1 . Further more, this will return an error if the provided expression is not a valid RulEx expression.","ref":"RulEx.Encoding.Json.html#encode!/1","title":"RulEx.Encoding.Json.encode!/1","type":"function"},{"doc":"Provide helper guards for use with RulEx.","ref":"RulEx.Guards.html","title":"RulEx.Guards","type":"module"},{"doc":"Yield true if given expression is a valid comparison expression, false otherwise.","ref":"RulEx.Guards.html#is_comparison/1","title":"RulEx.Guards.is_comparison/1","type":"macro"},{"doc":"Yield true if given operand is a valid comparison operand, false otherwise.","ref":"RulEx.Guards.html#is_comparison_operand/1","title":"RulEx.Guards.is_comparison_operand/1","type":"macro"},{"doc":"Yield true if given Elixir values is falsy, i.e. nil or false , false otherwise.","ref":"RulEx.Guards.html#is_falsy/1","title":"RulEx.Guards.is_falsy/1","type":"macro"},{"doc":"Yield true if given operand is reserved by RulEx, false otherwise.","ref":"RulEx.Guards.html#is_reserved_operand/1","title":"RulEx.Guards.is_reserved_operand/1","type":"macro"},{"doc":"Yield true if given Elixir values is truthy, i.e. not nil or false , false otherwise.","ref":"RulEx.Guards.html#is_truthy/1","title":"RulEx.Guards.is_truthy/1","type":"macro"},{"doc":"Yield true if given expression is a valid val expression, false otherwise.","ref":"RulEx.Guards.html#is_val/1","title":"RulEx.Guards.is_val/1","type":"macro"},{"doc":"Yield true if given expression is a valid val or var expression, false otherwise.","ref":"RulEx.Guards.html#is_val_or_var/1","title":"RulEx.Guards.is_val_or_var/1","type":"macro"},{"doc":"Yield true if give operand is a reserved operand or a string standing for any custom operands, false otherwise.","ref":"RulEx.Guards.html#is_valid_operand/1","title":"RulEx.Guards.is_valid_operand/1","type":"macro"},{"doc":"Yield true if given expression is a valid var expression, false otherwise.","ref":"RulEx.Guards.html#is_var/1","title":"RulEx.Guards.is_var/1","type":"macro"},{"doc":"An error risen if the given encoded RulEx expression fails to be decoded for any reason when using the bang variant functions of the RulEx.Encoding callbacks.","ref":"RulEx.DecodeError.html","title":"RulEx.DecodeError","type":"exception"},{"doc":"An error risen if the given RulEx expression fails to be encoded for any reason when using the bang variant functions of the RulEx.Encoding callbacks.","ref":"RulEx.EncodeError.html","title":"RulEx.EncodeError","type":"exception"},{"doc":"An error risen if the given RulEx expression fails to evaluate for any reason when using the bang variant functions of the RulEx.Behaviour callbacks.","ref":"RulEx.EvalError.html","title":"RulEx.EvalError","type":"exception"},{"doc":"A simple to use, simple to extend rules engine, written in Elixir.","ref":"readme.html","title":"RulEx","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding rul_ex to your list of dependencies in mix.exs : def deps do [ { :rul_ex , &quot;~&gt; 1.0.0&quot; } , # In order to use the default encoding, if you don&#39;t wish to use Jason # you can use any module you want that implements the same APIs as # Jason and configure the application environment [:rul_ex, RulEx.Encoding.Json] # configuration. { :jason , &quot;~&gt; 1.2&quot; } ] end Documentation can be generated with ExDoc and published on HexDocs . Once published, the docs can be found at https://hexdocs.pm/rul_ex . Note to anyone wanting to use this: If I would've needed this in a real life example I would probabl use it, however, this is not battle-tested, and was conceived just as means to have a fun side thing to do that didn't consume too much time/effort from me.","ref":"readme.html#installation","title":"RulEx - Installation","type":"extras"}]